@@ -75,7 +75,6 @@ class TCPServer:
                     alpn_protocol,
                 )
                 await self.protocol.initiate()
-                task_group.spawn(self.protocol.send_task())
                 await self._update_keep_alive_timeout()
                 await self._read_data()
         except OSError:
@@ -108,6 +107,7 @@ class TCPServer:
                 ConnectionResetError,
                 OSError,
                 asyncio.TimeoutError,
+                TimeoutError,
             ):
                 await self.protocol.handle(Closed())
                 break
@@ -134,6 +134,11 @@ class TCPServer:
         async with self.timeout_lock:
             if self._keep_alive_timeout_handle is not None:
                 self._keep_alive_timeout_handle.cancel()
+                try:
+                    await self._keep_alive_timeout_handle
+                except asyncio.CancelledError:
+                    pass
+
             self._keep_alive_timeout_handle = None
             if self.protocol.idle:
                 self._keep_alive_timeout_handle = self.loop.create_task(
